import os
from typing import Iterator, Optional

from authlib.integrations.httpx_client import AsyncOAuth2Client, OAuth2Client
from authlib.oauth2.rfc6749 import OAuth2Token

DEFAULT_BASE_URL = "https://api.iris.dioptra.io"
LOGIN_URL = "/auth/jwt/login"

USERNAME_ENV = "IRIS_USERNAME"
PASSWORD_ENV = "IRIS_PASSWORD"

PAGINATION_DATA_KEY = "results"
PAGINATION_NEXT_KEY = "next"

__version__ = "0.2.1"


class IrisClient(OAuth2Client):
    def __init__(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
        base_url: str = DEFAULT_BASE_URL,
        fetch_token: bool = True,
        **kwargs,
    ):
        super().__init__(base_url=base_url, **kwargs)
        self.username = username or os.environ.get(USERNAME_ENV)
        self.password = password or os.environ.get(PASSWORD_ENV)
        self.fetch_token_ = fetch_token

    def __enter__(self):
        super().__enter__()
        if self.fetch_token_:
            self.fetch_token()
        return self

    def fetch_token(self, **kwargs) -> OAuth2Token:
        return super().fetch_token(
            LOGIN_URL, username=self.username, password=self.password
        )

    def all(self, url: str, **kwargs) -> Iterator[dict]:
        while url:
            data = self.get(url, **kwargs).json()
            url = data[PAGINATION_NEXT_KEY]
            yield from data[PAGINATION_DATA_KEY]


class AsyncIrisClient(AsyncOAuth2Client):
    def __init__(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
        base_url: str = DEFAULT_BASE_URL,
        fetch_token: bool = True,
        **kwargs,
    ):
        super().__init__(base_url=base_url, **kwargs)
        self.username = username or os.environ.get(USERNAME_ENV)
        self.password = password or os.environ.get(PASSWORD_ENV)
        self.fetch_token_ = fetch_token

    async def __aenter__(self):
        await super().__aenter__()
        if self.fetch_token_:
            await self.fetch_token()
        return self

    async def fetch_token(self, **kwargs) -> OAuth2Token:
        return await super().fetch_token(
            LOGIN_URL, username=self.username, password=self.password
        )

    async def all(self, url: str, **kwargs) -> Iterator[dict]:
        while url:
            data = (await self.get(url, **kwargs)).json()
            url = data[PAGINATION_NEXT_KEY]
            for obj in data[PAGINATION_DATA_KEY]:
                yield obj



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['iris_client']
install_requires = \
['Authlib>=0.15.5,<0.16.0', 'httpx>=0.22.0,<0.23.0']

setup_kwargs = {
    'name': 'dioptra-iris-client',
    'version': '0.2.1',
    'description': 'Python client for the Iris API.',
    'long_description': '# 🕸️ Iris Python Client\n\n[![Tests](https://img.shields.io/github/workflow/status/dioptra-io/iris-client/Tests?logo=github)](https://github.com/dioptra-io/iris-client/actions/workflows/tests.yml)\n[![Coverage](https://img.shields.io/codecov/c/github/dioptra-io/iris-client?logo=codecov&logoColor=white)](https://app.codecov.io/gh/dioptra-io/iris-client)\n[![PyPI](https://img.shields.io/pypi/v/dioptra-iris-client?logo=pypi&logoColor=white)](https://pypi.org/project/dioptra-iris-client/)\n\nMinimal Python client for the [Iris](https://github.com/dioptra-io/iris) API,\nbuilt on top of [Authlib](https://github.com/lepture/authlib) and [httpx](https://github.com/encode/httpx).\n\n## Installation\n\n```bash\npip install dioptra-iris-client\n```\n\n## Usage\n\n```python\nfrom iris_client import IrisClient, AsyncIrisClient\n\n# NOTE: If the username and/or the password are not specified,\n# they will be retrieved from the `IRIS_USERNAME` and `IRIS_PASSWORD` environment variables.\n\n# Synchronous client\nwith IrisClient("user@example.org", "password") as client:\n    measurements = client.get("/measurements/").json()\n\n# Asynchronous client\nasync with AsyncIrisClient("user@example.org", "password") as client:\n    measurements = (await client.get("/measurements/")).json()\n\n# Helper function to fetch all the results from a paginated endpoint,\n# available for both clients:\nall_measurements = client.all("/measurements/")\n```\n',
    'author': 'Maxime Mouchet',
    'author_email': 'maxime.mouchet@lip6.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dioptra-io/iris-client',
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
